SystemOrganization addCategory: #Printf!

!Object methodsFor: '*printf' stamp: 'eem 11/2/2017 14:00'!
asArgumentArrayForFormatString

	^{self}! !

!Object methodsFor: '*printf' stamp: 'eem 11/2/2017 14:15'!
printLeanString
	^self printStringLimitedTo: 256! !

Object subclass: #PrintfFormatDescriptor
	instanceVariableNames: 'flush width precision'
	classVariableNames: 'Flags Operators'
	poolDictionaries: ''
	category: 'Printf'!

PrintfFormatDescriptor subclass: #PrintfCharacterFormatDescriptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

!PrintfCharacterFormatDescriptor methodsFor: 'rendering' stamp: ''!
applyOperator: object
	^ String with: object asCharacter! !

!PrintfCharacterFormatDescriptor methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	self printWidthOn: aStream.
	aStream nextPut: $c! !

!PrintfFormatDescriptor class methodsFor: 'class initialization' stamp: 'eem 11/2/2017 14:28'!
initialize
	"PrintfFormatDescriptor initialize"
	Operators := Dictionary new.
	Operators at: $P put: #PrintStringFormatDescriptor.
	Operators at: $c put: #PrintfCharacterFormatDescriptor.
	Operators at: $s put: #PrintfStringFormatDescriptor.
	#($d $o $u $x $X $p)
		do: [:operator | Operators at: operator put: #PrintfNumberFormatDescriptor].
	#($e $E $f $g $G $a $A)
		do: [:operator | Operators at: operator put: #PrintfFloatFormatDescriptor].

	Flags := Dictionary new.
	Flags at: $- put: #leftFlush.
	Flags at: $+ put: #rightFlush.
	Flags at: $  put: #space.
	Flags at: $# put: #radix.
	Flags at: $0 put: #zero! !

!PrintfFormatDescriptor class methodsFor: 'instance creation' stamp: ''!
new
	^ super new initialize! !

!PrintfFormatDescriptor class methodsFor: 'instance creation' stamp: ''!
newFrom: desc
	| myself |
	myself := self new.
	myself perform: desc flush.
	myself width: desc width.
	myself precision: desc precision.
	^ myself! !

!PrintfFormatDescriptor class methodsFor: 'instance creation' stamp: 'eem 1/7/2021 15:05'!
scanFrom: stream
	| descriptor numberHolderDescriptor |
	descriptor := self new.
	[Flags includesKey: stream peek]
		whileTrue: [descriptor := descriptor perform: (Flags at: stream next)].
	stream peek isDigit ifTrue: [descriptor width: (Integer readFrom: stream)].
	stream peek == $. ifTrue:
		[stream next.
		 stream peek == $*
			ifTrue: [numberHolderDescriptor := PrintfNumberHolderDescriptor new. stream next]
			ifFalse: [descriptor precision: (Integer readFrom: stream)]].
	stream peek == $l ifTrue: [stream next].
	descriptor := descriptor operator: stream next.
	numberHolderDescriptor ifNotNil:
		[descriptor widthDescriptor: numberHolderDescriptor.
		 ^{numberHolderDescriptor. descriptor}].
	^descriptor! !

!PrintfFormatDescriptor methodsFor: 'rendering' stamp: ''!
applyOperator: object
	self subclassResponsibility! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
flush
	^ flush! !

!PrintfFormatDescriptor methodsFor: 'initialize-release' stamp: 'nice 9/28/2021 00:51'!
initialize
	flush := #yourself.
	width := 0! !

!PrintfFormatDescriptor methodsFor: 'scanning' stamp: ''!
leftFlush
	flush := #leftFlush! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: 'mir 6/6/2000 23:58'!
operator: char
	| myself |
	myself := (Smalltalk at: (Operators at: char)) newFrom: self.
	myself setOperator: char.
	^ myself! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
padding
	^ Character space! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
precision
	^ precision! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
precision: anInteger
	precision := anInteger! !

!PrintfFormatDescriptor methodsFor: 'printing' stamp: 'nice 9/28/2021 00:50'!
printOn: aStream
	aStream nextPut: $%.
	flush == #leftFlush ifTrue: [aStream nextPut: $-].
	flush == #leftFlush ifTrue: [aStream nextPut: $+]! !

!PrintfFormatDescriptor methodsFor: 'printing' stamp: 'eem 1/7/2021 15:10'!
printWidthOn: aStream
	width ~= 0 ifTrue: [width printOn: aStream].
	precision ifNotNil: [aStream nextPut: $.. precision printOn: aStream]! !

!PrintfFormatDescriptor methodsFor: 'scanning' stamp: 'eem 11/2/2017 14:17'!
radix
	^ (PrintfNumberFormatDescriptor newFrom: self) radix! !

!PrintfFormatDescriptor methodsFor: 'rendering' stamp: 'eem 9/23/2021 22:53'!
render: object for: printfFormatString
	| string |
	string := self applyOperator: object.
	self stringLength ~= 0 ifTrue:
		[ string := string copyFrom: 1 to: (self stringLength min: string size) ].
	width = 0 ifTrue: [ ^ string ].
	^(String new: width withAll: self padding)
		copyReplaceFrom: (self startIndexOfCopyReplaceWithStringSize: string size)
		to: (self stopIndexOfCopyReplaceWithStringSize: string size)
		with: string! !

!PrintfFormatDescriptor methodsFor: 'scanning' stamp: ''!
rightFlush
	flush := #rightFlush! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
setOperator: char! !

!PrintfFormatDescriptor methodsFor: 'scanning' stamp: 'eem 11/2/2017 14:17'!
space
	^ (PrintfNumberFormatDescriptor newFrom: self) space! !

!PrintfFormatDescriptor methodsFor: 'rendering' stamp: 'nice 9/28/2021 00:49'!
startIndexOfCopyReplaceWithStringSize: anInteger

	|start|
	start := flush == #leftFlush
		ifTrue: [1]
		ifFalse: [width - anInteger + 1]. 
	^(start max: 1)
! !

!PrintfFormatDescriptor methodsFor: 'rendering' stamp: 'nice 9/28/2021 00:50'!
stopIndexOfCopyReplaceWithStringSize: anInteger

	| stop |
	stop := flush == #leftFlush
		ifTrue: [anInteger]
		ifFalse: [width]. 
	^stop min: width! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: 'eem 9/24/2021 07:20'!
stringLength
	^precision ifNil: [0]! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
width
	^ width! !

!PrintfFormatDescriptor methodsFor: 'private' stamp: ''!
width: anInteger
	width := anInteger! !

!PrintfFormatDescriptor methodsFor: 'scanning' stamp: 'eem 11/2/2017 14:17'!
zero
	^ (PrintfNumberFormatDescriptor newFrom: self) zero! !

PrintfFormatDescriptor subclass: #PrintfNumberFormatDescriptor
	instanceVariableNames: 'operator padding radix space plusSign'
	classVariableNames: 'Base Cased Radix'
	poolDictionaries: ''
	category: 'Printf'!

PrintfNumberFormatDescriptor subclass: #PrintfFloatFormatDescriptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

!PrintfFloatFormatDescriptor methodsFor: 'rendering' stamp: 'nice 9/28/2021 01:54'!
applyOperator: aNumber 
	"Number is the only valid object class.
	Emulate C behavior as much as we understand it"

	| printer |
	printer := ('gG' includes: operator)
		ifTrue: [FloatPrinter freeFormat]
		ifFalse: [('eE' includes: operator)
			ifTrue: [FloatPrinter scientific]
			ifFalse: [('fF' includes: operator)
				ifTrue: [FloatPrinter fixed]
				ifFalse: [('aA' includes: operator)
					ifTrue: [FloatPrinter hex]
					ifFalse: [self error: 'unknown print operator']]]].
	printer actAsPrintf.
	printer digitCount: (precision ifNil: [6]).
	printer positiveSignChar: plusSign.
	^flush == #leftFlush
		ifTrue:
			["leftFLush is not handled by versatile printer yet, do it ourself"
			| print |
			print := String new: 32 streamContents: [:str | printer print: aNumber on: str].
			"Note that zero padding is ignored in leftFlush mode. Always use space padding"
			width ifNotNil: [width > print size ifTrue: [print := print , (String new: width - print size withAll: Character space)]].
			print]
		ifFalse:
			[width ifNotNil: [printer totalWidth: width].
			padding = $0 ifTrue: [printer padWithZeros: true].
			String new: 32 streamContents: [:str | printer print: aNumber on: str]].! !

!PrintfFloatFormatDescriptor methodsFor: 'rendering' stamp: 'nice 9/28/2021 01:52'!
render: object for: printfFormatString
	"Bypass super, because we don't want it to truncate to prescribed width"
	^self applyOperator: object.! !

!PrintfFloatFormatDescriptor methodsFor: 'private' stamp: 'nice 9/27/2021 13:37'!
stringLength
	"answer zero because width and padding are handled locally"
	^0! !

!PrintfNumberFormatDescriptor class methodsFor: 'class initialization' stamp: 'eem 11/2/2017 15:17'!
initialize
	"PrintfNumberFormatDescriptor initialize"
	Base := Dictionary newFromPairs: #(	$d 10
											$o 8
											$p 16
											$u 10
											$x 16
											$X 16).

	Radix := Dictionary newFromPairs: #(	$d ''
											$o '0'
											$p '0x'
											$u ''
											$x '0x'
											$X '0X').

	Cased := 'AaEeFfGgXx'! !

!PrintfNumberFormatDescriptor class methodsFor: 'instance creation' stamp: ''!
newFrom: desc
	desc class == self ifTrue: [^ desc].
	^ (super newFrom: desc) setPadding: desc padding! !

!PrintfNumberFormatDescriptor methodsFor: 'rendering' stamp: 'nice 9/28/2021 01:57'!
applyOperator: object 
	"Character and Number are the only valid classes"

	| number string |
	object ifNil: [^'-'].
	number := object asInteger.
	string := number printStringBase: self base.
	(radix or: [operator == $p]) ifTrue: [string := self radixString , string].
	(Cased includes: operator) ifTrue:
		[string := operator isLowercase
					ifTrue: [string asLowercase]
					ifFalse: [string asUppercase]].
	(plusSign notNil and: [operator == $d and: [number >= 0]]) ifTrue:
		[string := (String with: plusSign) , string].
	^ (width ~= 0 and: [string size > self stringLength])
		ifTrue: [String new: width withAll: $*]
		ifFalse: [string]! !

!PrintfNumberFormatDescriptor methodsFor: 'private' stamp: ''!
base
	^ Base at: operator! !

!PrintfNumberFormatDescriptor methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	padding := $ .
	radix := false.
	space := false! !

!PrintfNumberFormatDescriptor methodsFor: 'private' stamp: ''!
padding
	^ padding! !

!PrintfNumberFormatDescriptor methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	padding == $0 ifTrue: [aStream nextPut: $0].
	radix ifTrue: [aStream nextPut: $#].
	space ifTrue: [aStream nextPut: $ ].
	self printWidthOn: aStream.
	aStream nextPut: operator! !

!PrintfNumberFormatDescriptor methodsFor: 'scanning' stamp: ''!
radix
	radix := true! !

!PrintfNumberFormatDescriptor methodsFor: 'private' stamp: ''!
radixString
	^ Radix at: operator! !

!PrintfNumberFormatDescriptor methodsFor: 'scanning' stamp: 'nice 9/28/2021 00:16'!
rightFlush
	plusSign := $+.
	super rightFlush! !

!PrintfNumberFormatDescriptor methodsFor: 'private' stamp: ''!
setOperator: char
	operator := char! !

!PrintfNumberFormatDescriptor methodsFor: 'private' stamp: ''!
setPadding: paddingChar
	padding := paddingChar! !

!PrintfNumberFormatDescriptor methodsFor: 'scanning' stamp: 'nice 9/28/2021 01:56'!
space
	space := true.
	plusSign := Character space! !

!PrintfNumberFormatDescriptor methodsFor: 'private' stamp: 'eem 9/24/2021 15:11'!
stringLength
	^precision
		ifNil: [SmallInteger maxVal]
		ifNotNil: [precision]! !

!PrintfNumberFormatDescriptor methodsFor: 'scanning' stamp: ''!
zero
	padding := $0! !

PrintfFormatDescriptor subclass: #PrintfNumberHolderDescriptor
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

!PrintfNumberHolderDescriptor methodsFor: 'rendering' stamp: 'eem 1/7/2021 15:13'!
applyOperator: ignored! !

!PrintfNumberHolderDescriptor methodsFor: 'rendering' stamp: 'eem 9/24/2021 14:08'!
render: object for: printfFormatString
	^printfFormatString nextFormat
		render: (printfFormatString nextArgument copyFrom: 1 to: object)
		for: printfFormatString! !

!PrintfNumberHolderDescriptor methodsFor: 'accessing' stamp: 'eem 1/7/2021 15:14'!
value
	^value! !

PrintfFormatDescriptor subclass: #PrintfStringFormatDescriptor
	instanceVariableNames: 'widthDescriptor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

PrintfStringFormatDescriptor subclass: #PrintStringFormatDescriptor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

!PrintStringFormatDescriptor methodsFor: 'rendering' stamp: ''!
applyOperator: object
	^ object printLeanString! !

!PrintStringFormatDescriptor methodsFor: 'printing' stamp: ''!
printOn: aStream
	aStream nextPut: $%.
	flush == #leftFlush ifTrue: [aStream nextPut: $-].
	self printWidthOn: aStream.
	aStream nextPut: $p! !

!PrintfStringFormatDescriptor methodsFor: 'rendering' stamp: ''!
applyOperator: object
	^ object! !

!PrintfStringFormatDescriptor methodsFor: 'printing' stamp: ''!
printOn: aStream
	super printOn: aStream.
	self printWidthOn: aStream.
	aStream nextPut: $s! !

!PrintfStringFormatDescriptor methodsFor: 'printing' stamp: 'eem 1/7/2021 15:11'!
printWidthOn: aStream
	widthDescriptor ifNotNil: [aStream nextPutAll: '.*']! !

!PrintfStringFormatDescriptor methodsFor: 'accessing' stamp: 'eem 9/24/2021 13:16'!
stringLength
	^widthDescriptor
		ifNotNil: [:wd| wd value ifNil: [0]]
		ifNil: [super stringLength]! !

!PrintfStringFormatDescriptor methodsFor: 'accessing' stamp: 'eem 1/7/2021 15:02'!
widthDescriptor: aPrintfNumberFormatDescriptor

	widthDescriptor := aPrintfNumberFormatDescriptor! !

Object subclass: #PrintfFormatString
	instanceVariableNames: 'format string outputStream arguments total'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

!PrintfFormatString commentStamp: 'mir 6/7/2000 00:14' prior: 0!
Format description 
	syntax: %{flags}{width}{precision}{long}<operator> 
	
	flags 
		-		left flush 
		+		right flush 
		space	non-negative number are preceeded by a blank 
		#		display integer with a radix indicator (0=octal, 0x=hex, float have .) 
		0		0 is used as left padding character for numbers 
	width		minimum field width (rest is padded) 
	.precision	maximum field width or trailing digits 
	long		ignored 
	operator 
		c		display object as character 
		d		display as integer 
		e,E		float in scientific notation 
		f		display as float 
		g,G		display as f or e,E using least amount of space 
		o		display as octal value 
		s		display as string 
		u		display as unsigned 
		x,X		display as hex value 
	!

!PrintfFormatString class methodsFor: 'examples' stamp: 'mir 6/7/2000 00:12'!
examples
	self inform: ('Here is a string "%s".' printf: 'hello world').
	self inform: ('Here is a string "%s" and another shortened "%-14.7s".'
		printf: #('hello world' 'hello world')).

	self inform: ('Here is a number "%d".' printf: 42).
	self inform: ('Here is a string "%07.7d".' printf: 42).

	self inform: ('Here is a number "%e".' printf: 42.0).
	self inform: ('Here is a float "%e" and an integer "%d".' printf: #(42.0 42)).
	self inform: ('Here is a string "%013.5e".' printf: 42.1234567).

	self inform: ('Here is a %s string "%s" and the same shortened "%-14.7s" with left flush.\nThe new line has a number "%e" and a 0-padded limited precision one "%013.5e".'
		printf: ((Array with: 'long' with: 'hello world' with: 'hello world' with: 42.0) copyWith: 42.1234567)).! !

!PrintfFormatString methodsFor: 'printf' stamp: 'eem 9/23/2021 22:54'!
<< object 
	"Render object according to next format descriptor in format. 
	Append it to string"
	"Format description 
	syntax: %{flags}{width}{precision}{long}<operator> 
	
	flags 
		-		left flush 
		+		right flush 
		space	non-negative number are preceeded by a blank 
		#		display integer with a radix indicator (0=octal, 0x=hex, float have .) 
		0		0 is used as left padding character for numbers 
	width		minimum field width (rest is padded) 
	.precision	maximum field width or trailing digits 
	long		ignored 
	operator 
		c		display object as character 
		d		display as integer 
		e,E		float in scientific notation 
		f		display as float 
		g,G		display as f or e,E using least amount of space 
		o		display as octal value 
		s		display as string 
		u		display as unsigned 
		x,X		display as hex value 
	"

	outputStream nextPutAll: string next.
	outputStream nextPutAll: (format next render: object for: self).
	format atEnd ifTrue: 
		[format reset.
		outputStream nextPutAll: string next.
		string reset].
	^self string! !

!PrintfFormatString methodsFor: 'private' stamp: 'eem 1/7/2021 14:55'!
collectFormatDescriptorsAndStrings: formatStream
	format := ReadWriteStream on: (Array new: 10).
	string := ReadWriteStream on: (Array new: 10).
	[| descriptor |
	 "copy actual formatstrings to format"
	 string nextPut: (self scanStringFrom: formatStream).
	 formatStream atEnd ifTrue:
		[self reset.
		 ^self].
	 descriptor := PrintfFormatDescriptor scanFrom: formatStream.
	 "copy any nonformating text to string"
	 descriptor isArray
		ifTrue: [format nextPutAll: descriptor]
		ifFalse: [format nextPut: descriptor]] repeat! !

!PrintfFormatString methodsFor: 'printf' stamp: 'eem 9/24/2021 11:45'!
f: aStream printf: arguments
	"inst var string holds all text contained in the formatstring. %f blabla %d"
	| savedStream |
	savedStream := outputStream.
	outputStream := aStream.
	self reset.
	^[self process: arguments]
		ensure: [outputStream := savedStream]! !

!PrintfFormatString methodsFor: 'accessing' stamp: 'eem 9/24/2021 07:15'!
nextArgument
	^arguments next! !

!PrintfFormatString methodsFor: 'accessing' stamp: 'eem 9/24/2021 14:09'!
nextFormat
	^format next! !

!PrintfFormatString methodsFor: 'private' stamp: 'eem 9/24/2021 11:29'!
output: aString
	| code length |
	length := aString size.
	length > 0 ifTrue:
		[total := total + length.
		 outputStream nextPutAll: aString.
		 code := aString basicAt: length.
		 (code = 10 or: [code = 13]) ifTrue:
			[outputStream flush]]! !

!PrintfFormatString methodsFor: 'printf' stamp: 'eem 9/24/2021 11:46'!
printf: arguments
	"inst var string holds all text contained in the formatstring. %f blabla %d"

	"inst var format is a stream of FormatDescriptors"
	self reset.
	^self process: arguments! !

!PrintfFormatString methodsFor: 'private' stamp: 'eem 9/24/2021 15:46'!
process: argumentOrArgumentArray
 	arguments := ReadStream on: argumentOrArgumentArray asArgumentArrayForFormatString.
	[arguments atEnd] whileFalse:
		["put any text from the format string into outputStream"
		self output: string next. 
		"get next FormatDescriptor from format and render object as specified"
		format atEnd ifFalse: [self output: (format next render: arguments next for: self)]].
	"any remainder is string, if so append to outputStream"
	string atEnd ifFalse: [self output: string next].
	^total! !

!PrintfFormatString methodsFor: 'private' stamp: 'eem 9/24/2021 11:30'!
reset
	format reset.
	string reset.
	outputStream reset.
	total := 0! !

!PrintfFormatString methodsFor: 'private' stamp: 'eem 9/24/2021 13:05'!
scanStringFrom: aStream 
	| newString |
	newString := (String new: 40) writeStream.
	[aStream atEnd] whileFalse: 
		[| next | 
		next := aStream next.
		next == $% ifTrue: [^newString contents].
		next == $\ ifTrue: 
				[next := aStream next.
				next == $n ifTrue: [next := Character cr].
				next == $t ifTrue: [next := Character tab].
				next == $r ifTrue: [next := Character cr].
				next == $\ ifTrue: [next := $\]].
		newString nextPut: next].
	^newString contents! !

!PrintfFormatString methodsFor: 'initialize-release' stamp: 'eem 9/25/2021 16:07'!
setFormat: aFormatString
	outputStream := (String new: (aFormatString size max: 20)) writeStream.
	self collectFormatDescriptorsAndStrings: aFormatString readStream! !

!PrintfFormatString methodsFor: 'initialize-release' stamp: 'eem 9/24/2021 13:10'!
setFormat: aFormatString output: aStream
	"copy actual formatstrings to format"
	"copy any nonformating text to string"
	outputStream := aStream.
	self collectFormatDescriptorsAndStrings: aFormatString readStream! !

!PrintfFormatString methodsFor: 'printf' stamp: 'eem 11/2/2017 15:32'!
string
	^outputStream contents! !

!WriteStream methodsFor: '*Printf-printing' stamp: 'eem 9/24/2021 15:20'!
f: printfString printf: arguments
	"Use the printf convention to output on the receiver."
	^(PrintfFormatString new setFormat: printfString output: self) printf: arguments! !

!WriteStream methodsFor: '*Printf-printing' stamp: 'eem 9/24/2021 15:21'!
fprintf: printfString
	"Use the printf convention to output on the receiver."
	^(PrintfFormatString new setFormat: printfString output: self) printf: #()! !

!NullStream methodsFor: '*Printf-printing' stamp: 'eem 9/24/2021 16:59'!
f: printfString printf: arguments! !

!NullStream methodsFor: '*Printf-printing' stamp: 'eem 9/24/2021 16:59'!
fprintf: printfString! !

!String methodsFor: '*printf' stamp: 'eem 11/2/2017 14:00'!
asArgumentArrayForFormatString

	^{self}! !

!String methodsFor: '*printf' stamp: 'eem 9/24/2021 11:58'!
f: stream printf: arguments
	"Answer the string that results from applying the receiver
	 as a printf-style format string to the arguments.  For example
		'Hello %d world!!' printf: {Date today year}"
	^(PrintfFormatString new setFormat: self output: stream) printf: arguments! !

!String methodsFor: '*printf' stamp: 'eem 11/2/2017 13:59'!
printf
	^ (self indexOf: $%) ~= 0
		ifTrue: [ self ]
		ifFalse: [ (PrintfFormatString new setFormat: self) printf: #() ]! !

!String methodsFor: '*printf' stamp: 'eem 11/10/2017 10:51'!
printf: arguments
	"Answer the string that results from applying the receiver
	 as a printf-style format string to the arguments.  For example
		'Hello %d world!!' printf: {Date today year}"
	^(PrintfFormatString new setFormat: self)
		printf: arguments;
		string! !

!Collection methodsFor: '*printf' stamp: 'hjo 9/17/2011 23:47'!
asArgumentArrayForFormatString

	^self! !

TestCase subclass: #PrintfFormatStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Printf'!

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'nice 9/28/2021 01:17'!
testFprintf
	| stream expected |
	stream := String new writeStream.
	expected := 'Here is a long string "hello world" and the same shortened "hello w       " with left flush.\The new line has a number "4.200000e+01"\and a 0-padded limited precision one "0000042.12346".' withCRs.
	self assert: expected size equals: (stream f: 'Here is a %s string "%s" and the same shortened "%-14.7s" with left flush.\nThe new line has a number "%e"
and a 0-padded limited precision one "%013.5f".'
		printf: #('long' 'hello world' 'hello world' 42.0 42.1234567)).
	self assert: expected equals: stream contents! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'nice 9/27/2021 23:33'!
testManyArguments

	self assert: 'Here is a long string "hello world" and the same shortened "hello w       " with left flush.
The new line has a number "42" and a 0-padded limited precision one "0000042.12346".' equals: ('Here is a %s string "%s" and the same shortened "%-14.7s" with left flush.\nThe new line has a number "%g" and a 0-padded limited precision one "%013.5f".'
		printf: #('long' 'hello world' 'hello world' 42.0 42.1234567))! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'nice 9/28/2021 01:37'!
testOneExponent

	self assert: 'Here is a number "42".' equals: ('Here is a number "%g".' printf: 42).
	self assert: 'Here is a number "42.000000".' equals: ('Here is a number "%f".' printf: 42).
	self assert: 'Here is a number "4.200000e+01".' equals: ('Here is a number "%e".' printf: 42).! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'nice 9/27/2021 23:41'!
testOneExponentAndOneInteger

	self assert: 'Here is a float "42.000000" and an integer "42".' equals: ('Here is a float "%f" and an integer "%d".' printf: #(42.0 42)).! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'nice 9/27/2021 23:59'!
testOneExponentWithLength

	self assert: 'Here is a string "0000042.12346".' equals: ('Here is a string "%013.5f".' printf: 42.1234567).
	self assert: 'Here is a string "    +42.12346".' equals: ('Here is a string "%+13.5f".' printf: 42.1234567).
	self assert: 'Here is a string "42.12346     ".' equals: ('Here is a string "%-13.5f".' printf: 42.1234567).! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'hjo 9/17/2011 23:40'!
testOneInteger

	self assert: 'Here is a number "42".' equals: ('Here is a number "%d".' printf: 42).! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'hjo 9/17/2011 23:40'!
testOneIntegerWithLength

	self assert: 'Here is a string "0000042".' equals: ('Here is a string "%07.7d".' printf: 42).! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'eem 11/2/2017 14:38'!
testOnePrintPointer

	self assert: 'Here is a pointer "0x12345678".' equals: ('Here is a pointer "%p".' printf: 16r12345678)! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'eem 11/2/2017 14:30'!
testOnePrintString

	self assert: 'Here is a point "3@4".' equals: ('Here is a point "%P".' printf: 3@4)! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'hjo 9/17/2011 23:40'!
testOneString

	self assert: 'Here is a string "hello world".' equals: ('Here is a string "%s".' printf: 'hello world')! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'hjo 9/17/2011 23:40'!
testOneStringWithLength

	self assert:  'Here is a string "hello world" and another shortened "hello w       ".' equals: ('Here is a string "%s" and another shortened "%-14.7s".' printf: #('hello world' 'hello world'))! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'eem 9/24/2021 07:22'!
testOneStringWithVariableLength

	self assert: 'Hello world!!!!' equals: ('Hello %.*s!!!!' printf: #(5 'world world world'))! !

!PrintfFormatStringTest methodsFor: 'tests' stamp: 'eem 11/10/2017 10:41'!
testReturnValue

	self assert: 6+8 equals: ((PrintfFormatString new setFormat: 'hello %ld') printf: {12345678})! !
PrintfFormatDescriptor initialize!
PrintfNumberFormatDescriptor initialize!
